 /*
Research Project:   Graphical Database for Category Theory
                    J. Bradbury, Dr. R. Rosebrugh, I. Rutherford
                    Mount Allison University 2001
File:               CategoryModified.java
Description:        This is a class that contains information about modifications
                    to the current category. Modififications that are recorded
                    include:
                      -> GML Changed
                      -> Data Added
                      -> Data Removed
                      -> Category Name Changed
                      -> Relations Added through Make Confluent
                    The class also indicates if the files was not originally
                    saved. This could be the result of:
                      -> Download from Server
                      -> Created using "Create Category"
                      -> Created using "Make Dual"
                      -> Created as part of a functor
*/

public class CategoryModified
//class CategoryModified definition
{
	boolean dataAdded;      //Indicates if data has been added to the category
	boolean dataRemoved;    //Indicates if data has been removed from the category
	boolean nameChanged;    //Indicates if the name of the category has been changed
	boolean madeConfluent;  //Indicates if the category has been made confluent
	                          //and if relations were added to the category
	boolean downloadCat;    //Indicates if the category was downloaded from the
	                        //server and thus has not been saved on the local machine
	boolean dualCat;        //Indicates if the category was created as a dual category
	                        //and thus has not been saved on the local machine
	boolean createCat;      //Indicates if the category was created
	                        //and thus has not been saved on the local machine
	boolean functorCat;     //Indicates if the category was created as part of a functor
	                        //and thus has not been saved on the local machine
	boolean gmlChanged;     //Indicates if the graphical display has been changed


	public CategoryModified()
	// Default Constructor
	{
		//Initialize all variables to false
		dataAdded = false;
		dataRemoved = false;
		gmlChanged = false;
		nameChanged = false;
		downloadCat = false;
		madeConfluent = false;
		dualCat = false;
		createCat = false;
		functorCat = false;
	}

	public void isDownloaded()
	//Indicates that the current category was downloaded
	//from the server
	{
		downloadCat = true;
	}

	public void isCreated()
	//Indicates that the current category was created
	{
		createCat = true;
	}

	public void isDual()
	//Indicates that the current category was created
	//as a dual category using the GDCT "Make Dual" tool
	{
		dualCat = true;
	}

	public void isFunctorCat()
	//Indicates that the current category was created
	//as part of a functor and is not saved on its own as
	//a category
	{
		functorCat = true;
	}
}
