/*
Research Project:   Graphical Database for Category Theory
                    J. Bradbury, Dr. R. Rosebrugh, I. Rutherford
                    Mount Allison University 2001
File:               FunctorNode.java
Description:	      This class contains the information about a functor that is
                    later stored in a linked list.
*/
import java.io.*;

public class FunctorNode
//class FunctorNode definition
{
	CatFunctor f;
	FunctorNode next;
	Category A;
	Category B;
	Category diagram;
	int functor_type;
	int saved;
	File file_;

	public FunctorNode()
	//Default constructor for FunctorNode class
	{
		f = new CatFunctor();
		next = null;
		A = new Category();
		B = new Category();
		diagram = new Category();
		functor_type = 0;
		file_ = null;
	}

	public FunctorNode(Category newA, Category newB, Category newDiagram, CatFunctor newf)
	//Alternate constructor that is applicable when category A and B
	//along with the cat_functor are included as parameters
	{
		A = newA;
		B = newB;
		diagram = newDiagram;
		f = newf;
		next = null;
		file_ = null;
	}
	
	public boolean check_domain(int path[], int arrow)
	/*
	Parameters:
	path: the path in B that the arrow goes to under the functor
	arrow: the arrow of A that goes to the path under the functor
	f: a category functor
	A: the category that the functor goes from
	B: the category that the functor goes to
	Purpose:
	To check that the object which is the domain of the arrow in A goes to the
	object which is the domain of the path in B that the arrow goes to under the functor
	Design:
	Returns true if the domains match up, false otherwise
	*/
	{
		if(f.obj[A.arr[arrow][0]] != B.arr[path[B.path_len(path)-1]][0]) {return false;}
		return true;
	}

	public boolean check_codomain(int path[], int arrow)
	//This function checks that the object which is the codomain of the arrow in
	//A goes to the object which is the codomain of the path in B that the arrow goes
	//to under the functor. It returns true if the codomains match up, false otherwise
	{
		if(f.obj[A.arr[arrow][1]] != B.arr[path[0]][1]) {return(false);}
		return true;
	}
	
	public void updateDiagramCategory()
	/*
		This ftion creates a new diagram based on the categories A and B,
		and creates a new random graph for it
	*/
	{
		// the diagram will be a copy of category A:
		//diagram = new Category();
		diagram.copyCategory(A);

		// but the names of each object and arrow will be taken from category B:
		int i, j;
		for (i=0; i<A.num_objects; i++)
		{
			diagram.obj[i] = B.obj[f.obj[i]];
		}
		for (i=0; i<A.num_arrows; i++)
		{
			diagram.arrow[i] = B.path_to_string(f.arr[i]);
		}

		// Modify the GML string by replacing old labels with new ones
		int pos = 0, strt, last, next, num;
		boolean done;
		String s = diagram.gml, label, newlabel;
		while (pos < s.length())
		{
			// look for the "node" label
			if (s.charAt(pos) == 'n' && s.substring(pos).startsWith("node"))
			{
				done = false;
				while (pos < s.length() && !done)
				{
					// then look for the "label" label afterwards
					if (s.charAt(pos) == 'l' && s.substring(pos).startsWith("label"))
					{
						// figure out where the label itself is (eg. in the line:
						// 'label "A"' we want the "A") using strt and pos
						pos += 7;
						strt = pos;
						while (s.charAt(pos) != '"') pos++;
						label = s.substring(strt, pos);
						
						// now replace the old label with the new one
						if (!label.startsWith("relation"))
						{
							newlabel = B.obj[f.obj[A.object_number(label)]];
							s = s.substring(0, strt) + newlabel + s.substring(pos);
						}
						done = true;
					}
					pos++;
				}
			}
			// look for the "edge" label
			else if (s.charAt(pos) == 'e' && s.substring(pos).startsWith("edge"))
			{
				done = false;
				while (pos < s.length() && !done)
				{
					// then look for the "label" label afterwards
					if (s.charAt(pos) == 'l' && s.substring(pos).startsWith("label"))
					{
						// figure out the starting and ending positions of the label
						pos += 7;
						strt = pos;
						while (s.charAt(pos) != '"') pos++;
						label = s.substring(strt, pos);
	
				// now check each arrow label - multiple arrow labels
				// are seperated by commas in the string
						next = 0;
						num = 1;
						while (next < label.length())
						{
							if (label.charAt(next) == ',') num++;
							next++;
						}
						// now we know how many arrow labels there are, so we can
						// change them one at a time
						next = 0;
						newlabel = "";
						for (i=0; i<num; i++)
						{
							if (i != 0) newlabel += ",";
							last = next;
							if (i < num-1)
							{
								while (label.charAt(next) != ',') next++;
								newlabel += B.path_to_string(f.arr[A.arrow_number(label.substring(last, next))]);
								next++;
							}
							else
							{
								newlabel += B.path_to_string(f.arr[A.arrow_number(label.substring(last))]);
							}
						}
						
						// now replace the old label with the new one
						s = s.substring(0, strt) + newlabel + s.substring(pos);
						done = true;
					}
					pos++;
				}
			}
			pos++;
		}
		diagram.gml = s;
	}
}