/*
Research Project:   Graphical Database for Category Theory
                    J. Bradbury, Dr. R. Rosebrugh, I. Rutherford
                    Mount Allison University 2001
                    Updated: May 8th, 2003 (J. Tweedle)
File:               InitialObjectFrame.java
Description:	      This file is displayed when the user selects the "Initial
                    Object" menu option in the main window. The user
                    can select a category from a choice menu and then
                    chose to check if one or all of the categories are intial.
                    The information on each object tested will be displayed in
                    the text area.  The algorithm to perform the initial object
                    test was removed and put into the file InitialObject.java.
                    The obselete Borland libraries were removed.
*/

//import statements
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;

public class InitialObjectFrame extends JInternalFrame
{
	private Category cat;
	private CategoryInternalFrame categoryInternalFrame_;
	private InitialObject initObj;

//Objects in the Graphical User Interface(GUI)
	private DefaultListModel objectModel = new DefaultListModel();
	private JList objectList = new JList(objectModel);
	private JPanel buttonPanel = new JPanel();
	private JButton checkButton = new JButton();
	private JButton selectAllButton = new JButton();
	private JLabel objectLabel = new JLabel();
	private JTextField currentCategory = new JTextField(10);

	public InitialObjectFrame(CategoryInternalFrame categoryInternalFrame)
	//Default Constructor with the list of categories and the ok
	//button as parameters
	{
		super(categoryInternalFrame.getTitle() + ": Initial Objects", true, true, false, false);
		categoryInternalFrame_ = categoryInternalFrame;
		cat = categoryInternalFrame_.getCategory();
		initObj = new InitialObject(cat, "initial");

		objectList.setBackground(Color.white);
		objectList.setBorder(BorderFactory.createEtchedBorder());
		objectList.getSelectionModel().setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		for (int i=0; i<cat.num_objects; i++)
		{
			objectModel.addElement(cat.obj[i]);
		}
		
		checkButton.setText("Check Object(s)");

		BoxLayout boxLay = new BoxLayout(buttonPanel, BoxLayout.X_AXIS);
		buttonPanel.add(selectAllButton);
		buttonPanel.add(checkButton);

		JScrollPane objectListScrollPane = new JScrollPane(objectList);
		objectListScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		objectListScrollPane.setBorder(BorderFactory.createEtchedBorder());

		Container contentPane = getContentPane();
		contentPane.add(objectListScrollPane, BorderLayout.CENTER);
		contentPane.add(buttonPanel, BorderLayout.SOUTH);
		contentPane.add(objectLabel, BorderLayout.NORTH);
		
		selectAllButton.setText("Select All");
		selectAllButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				selectAllButton_actionPerformed(e);
			}
		});
		checkButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				checkButton_objectList_actionPerformed(e);
			}
		});
		objectLabel.setText("Select Object(s) to Check:");
	}

	private void selectAllButton_actionPerformed(ActionEvent e)
	//This function selects all objects
	{
		objectList.setSelectionInterval(0, objectModel.getSize()-1); 
	}

	private void checkButton_objectList_actionPerformed(ActionEvent e)
	{
		String viewbox = "";
		initObj.setOutput("");
		if (objectModel.isEmpty()) return;
		if (objectList.getSelectionModel().isSelectionEmpty())
		{
			JOptionPane.showInternalMessageDialog(this, "No objects selected.", "Error", JOptionPane.ERROR_MESSAGE);			
			return;
		}
		for (int i = 0; i < objectModel.getSize(); i++)
		{
			initObj.setOutput("");
			if (objectList.isSelectedIndex(i))
			{
				try
				{
					int editsub = 0;
					String temp = cat.obj[i];
					editsub = cat.object_number(temp);
					initObj.initial_object(editsub);
					viewbox+=initObj.getOutput();
				}
				catch (ArrayIndexOutOfBoundsException aie) {}
			}
		}
		objectList.clearSelection();
		JOptionPane.showInternalMessageDialog(this, viewbox, "Initial Objects", JOptionPane.PLAIN_MESSAGE);
	}
}