
  //package flgl.gui_util;


   import java.awt.*;
   import java.awt.event.*;
   import javax.swing.*;



   /**
    * A panel class with convenience functions.
    * A GridBagLayout is automatically used.
    **/
   public class LPanel extends JPanel
   {
      /**
       * Default element spacing.
       **/
      public int spacing = 8;
   
      public GridBagLayout layout;
      public GridBagConstraints constraints;
   
      public LPanel()
      {
         super();
      
         layout = new GridBagLayout();
         constraints = new GridBagConstraints();
         setLayout(layout);
      
         constraints.insets = new Insets(spacing, spacing, spacing,
            spacing);
         constraints.fill = GridBagConstraints.NONE;
         constraints.ipadx = constraints.ipady = 0;
         constraints.weightx = constraints.weighty = 1.0;
      }
   
   
      /**
       * Finish initialization.
       * Frees up any resources that are no longer needed (these
       * are minimal anyway).
       **/
      public void finish()
      {
         constraints = null;
         layout = null;
      }
   
   
   
   
   
      /**
       * Add a label and text that fill a line.
       **/
      public JTextField addLabelTextLine(String label, int columns)
      {
         addLabel(label, 1, 1, 0.0, 0.0, 0, 2);
         return addTextField("", columns, 0, 0, 1.0, 0.0, 1, 1);
      }
   
   
   
      /**
       * Add a left aligned label at the start of a line.
       **/
      public JLabel addLineLabel(String string, int width) {
         return addLineLabel_(string, width, false);
      }
   
   
      /**
       * Add a full-width centered label.
       **/
      public JLabel addCenteredLineLabel(String string, int width) {
         return addLineLabel_(string, width, true);
      }
   
      private JLabel addLineLabel_(String string, int width, boolean centered)
      {
         JLabel label;
      
         if(width == 0) {
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            constraints.insets.bottom = 0;
         }
         else {
            constraints.gridwidth = width;
            constraints.weightx = 0.0;
         }
      
         if(centered)
            constraints.anchor = GridBagConstraints.CENTER;
         else
            constraints.anchor = GridBagConstraints.WEST;
         constraints.insets.top = spacing;
         constraints.insets.left = spacing;
         constraints.insets.right = spacing;
         label = new JLabel(string);
         layout.setConstraints(label, constraints);
         add(label);
      
         if(width == 0)
            constraints.insets.top = 0;
         constraints.gridwidth = 1;
         constraints.insets.bottom = spacing;
         constraints.weightx = 1.0;
      
         return label;
      }
   
   
   
   
   
      private class ButtonPanel extends JPanel implements LayoutManager
      {
         public ButtonPanel(int spacing) {
            super();
            spacing_ = spacing;
            setLayout(this);
         }
      
      
         private int width_() {
            // Enforce a uniform size on the SizeableButton children.
            Component[] children = getComponents();
            Dimension size;
            int maxw = 0;
            for(int i = 0; i < children.length; i++) {
               if(children[i] instanceof JButton) {
                  size = children[i].getPreferredSize();
                  if(size.width > maxw)
                     maxw = size.width;
               }
            }
         
            return maxw;
         }
      
      
         private int spacing_;
         public synchronized void addLayoutComponent(String name,
         Component component) {
         }
      
         public synchronized void removeLayoutComponent(Component component) {
         }
      
         public synchronized Dimension preferredLayoutSize(Container parent) {
            Component[] children = parent.getComponents();
            Dimension size;
            Dimension psize = new Dimension(0, 0);
            int width = width_();
            for(int i = 0; i < children.length; i++) {
               size = children[i].getPreferredSize();
               if(size.height > psize.height)
                  psize.height = size.height;
               psize.width += width;
            }
            psize.width += spacing * (children.length - 1);
         
            return psize;
         }
      
         public synchronized Dimension minimumLayoutSize(Container parent) {
            Component[] children = parent.getComponents();
            Dimension size;
            Dimension msize = new Dimension(0, 0);
            int width = width_();
            for(int i = 0; i < children.length; i++) {
               size = children[i].getPreferredSize();
               if(size.height > msize.height)
                  msize.height = size.height;
               msize.width += width;
            }
         
            return msize;
         }
      
         public synchronized void layoutContainer(Container parent) {
            Dimension msize = minimumLayoutSize(parent);
            Dimension size = parent.getSize();
            int space = size.width - msize.width;
            int width = width_();
            Component[] children = parent.getComponents();
            if(children.length == 1) {
               children[0].setLocation(space / 2, 0);
               size = children[0].getPreferredSize();
               size.width = width;
               children[0].setSize(size);
            }
            else {
               double spacing = space / (children.length - 1);
               double x = 0.0;
               for(int i = 0; i < children.length; i++) {
                  children[i].setLocation((int)x, 0);
                  size = children[i].getPreferredSize();
                  size.width = width;
                  children[i].setSize(size);
                  x += size.width;
                  x += spacing;
               }
            }
         }
      
      }
   
   
   
   
      /**
       * Add a panel of evenly-spaced buttons.
       * gridwith is the width of the panel as a whole.
       * a gridwidth of zero will use REMAINDER.
       * listener can be null.
       **/
      public ButtonPanel addButtonPanel(String labels, int gridwidth,
      ActionListener listener)
      {
         return addButtonPanel(labels, gridwidth, 2.0, listener);
      }
   
   
      public ButtonPanel addButtonPanel(String labels, int gridwidth,
      double top_spacing_multiple, ActionListener listener)
      {
         ButtonPanel panel = new ButtonPanel(spacing);
      
         JButton button;
         int start = 0;
         while(start < labels.length()) {
            int end = labels.indexOf('~', start);
            if(end == -1)
               end = labels.length();
         
            button = new JButton(labels.substring(start, end));
            //panel_layout.setConstraints(button, panel_constraints);
            panel.add(button);
            if(listener != null)
               button.addActionListener(listener);
         
            start = end + 1;
         }
      
         constraints.insets.top = (int)(spacing * top_spacing_multiple);
         constraints.insets.left = spacing;
         constraints.fill = GridBagConstraints.HORIZONTAL;
         if(gridwidth == 0)
            constraints.gridwidth = GridBagConstraints.REMAINDER;
         else
            constraints.gridwidth = gridwidth;
      
         layout.setConstraints(panel, constraints);
         add(panel);
      
         constraints.insets.top = spacing;
      
         return panel;
      }
   
   
   
   
   
   
      public JTextField addTextField(int len, int width, int anchor,
      double weightx, double weighty, int fill, int shift)
      {
         JTextField textfield = new JTextField("", len);
         return (JTextField)(addComponent(textfield, width, anchor,
            weightx, weighty, fill, shift));
      }
   
   
      public JTextField addTextField(String text, int len, int width, int anchor,
      double weightx, double weighty, int fill, int shift) 
      {
         JTextField textfield = new JTextField(text, len);
         return (JTextField)(addComponent(textfield, width, anchor,
            weightx, weighty, fill, shift));
      }
   
   
   
      public JLabel addLabel(String string, int width, int anchor,
      double weightx, double weighty, int fill, int shift)
      {
         return (JLabel)(addComponent(new JLabel(string), width, anchor,
            weightx, weighty, fill, shift));
      }
   
   
   
      public JButton addButton(String string, int width, int anchor,
      double weightx, double weighty, int fill, int shift)
      {
         return (JButton)(addComponent(new JButton(string), width, anchor,
            weightx, weighty, fill, shift));
      }
   
   
      public JButton addButton(String string, int width, int anchor,
      double weightx, double weighty, int fill, int shift,
      ActionListener listener)
      
      {
         JButton button = (JButton)(addComponent(new JButton(string), width, anchor,
         weightx, weighty, fill, shift));
         button.addActionListener(listener);
         return button;
      }
   
   
   
      public JCheckBox addCheckbox(String string, ButtonGroup group,
      boolean state, int width, int anchor,
      double weightx, double weighty, int fill, int shift)
      {
         JCheckBox cb = new JCheckBox(string, state);
         if(group != null)
            group.add(cb);
         return (JCheckBox)(addComponent(cb, width, anchor, weightx,
            weighty, fill, shift));
      }
   
   
      public JCheckBox addCheckbox(String string, ButtonGroup group,
      boolean state, int width, int anchor,
      double weightx, double weighty, int fill, int shift,
      ItemListener listener)
      
      {
         JCheckBox cb = new JCheckBox(string, state);
         if(group != null)
            group.add(cb);
         cb.addItemListener(listener);
         return (JCheckBox)(addComponent(cb, width, anchor, weightx, weighty,
            fill, shift));
      }
   
   
      /**
       * width is gridwidth
       * anchor: -1 = WEST, 0 = CENTER, 1 = EAST
       * fill: 0 = NONE, 1 = HORIZONTAL, 2 = VERTICAL, 3 = BOTH
       * shift: 1 = flush left, 2 = flush right, 3 = no l/r margins
       **/
      public JComponent addComponent(JComponent component, int width,
      int anchor, double weightx, double weighty, int fill,
      int shift)
      {
         constraints.insets.left = spacing;
         constraints.insets.right = spacing;
         if(shift == 1 || shift == 3)
            constraints.insets.left = 0;
         if(shift == 2 || shift == 3)
            constraints.insets.right = 0;
      
         if(anchor < 0)
            constraints.anchor = GridBagConstraints.WEST;
         else if(anchor == 0)
            constraints.anchor = GridBagConstraints.CENTER;
         else
            constraints.anchor = GridBagConstraints.EAST;
      
         if(width == 0)
            constraints.gridwidth = GridBagConstraints.REMAINDER;
         else
            constraints.gridwidth = width;
      
         constraints.weightx = weightx;
         constraints.weighty = weighty;
      
         if(fill == 0)
            constraints.fill = GridBagConstraints.NONE;
         else if(fill == 1)
            constraints.fill = GridBagConstraints.HORIZONTAL;
         else if(fill == 2)
            constraints.fill = GridBagConstraints.VERTICAL;
         else
            constraints.fill = GridBagConstraints.BOTH;
      
         layout.setConstraints(component, constraints);
         add(component);
      
      
      
         if(width == 0) {
            constraints.insets.top = 0;
            constraints.insets.bottom = spacing;
         }
      
         return component;
      }
   
   }
