/*f
Research Project:   Graphical Database for Category Theory
                    J. Bradbury, Dr. R. Rosebrugh, I. Rutherford
                    Matt Graves, Jesse Tweedle
                    Mount Allison University 2003
                    
File:               MainWindow2.java
Description:	    
*/

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.beans.*;


public class MainWindow2 extends JFrame implements InternalFrameListener
{
	private JDesktopPane desktop;
	private JMenu window_menu;
	private JMenu recent_menu;
	private ButtonGroup window_group;

	private Vector listData;
	private URL funURL,fglURL,catURL,cglURL;
	private URL fileURL;
	private InputStream input;
	private DataInputStream dataInput;
	private JFileChooser fchooser;
	private IniSettings ini = new IniSettings();


	public MainWindow2()
	{
		super("GDCT");
		try {UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());}
		catch(Exception e) {System.out.println("Error setting native LAF: " + e);}
		//Make the big window be indented 50 pixels from each edge 
		//of the screen.
		int inset = 50;
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setBounds(inset, inset, 
		screenSize.width - inset*2, 
		screenSize.height-inset*2);

		//Quit this app when the big window closes.
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
				}
		});

		//Set up the GUI.
		desktop = new JDesktopPane(); //a specialized layered pane

		setContentPane(desktop);
		setJMenuBar(createMenuBar());

		//Make dragging faster:
		desktop.putClientProperty("JDesktopPane.dragMode", "outline");
		desktop.setBackground(Color.gray);
		
		
		
		
		// so using a 2D array works the same way.
//		int[][][] test = new int[3][5][5];
/*		int[][] test = new int[3][5];
		int[] alpha = new int[2];
		int[] beta = new int[3];
		int[] gamma = new int[5];
		for (int i=0; i<5; i++)
		{
			gamma[i] = i;
		}
		alpha[0] = 0; alpha[1] = 1;
		beta[0] = 0;
		beta[1] = 1;
		beta[2] = 2;
		for (int j=0; j<3; j++)
		{
			for (int i=0; i<5; i++)
			{
				System.out.println(test[j][i]);
			}
		}



		test[0] = alpha;
		test[1] = beta;
		test[2] = gamma;
		for (int j=0; j<test.length; j++)
		{
			for (int i=0; i<test[j].length; i++)
			{
				System.out.println(test[j][i]);
			}
		}*/
/*		for (int j=0; j<3; j++)
		{
			for (int i=0; i<10; i++)
			{
				System.out.println(test[j][i]);
			}
		}*/
	}

	protected JMenuBar createMenuBar()
	{
		JMenuBar menuBar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");
		window_menu = new JMenu("Window");
		window_menu.setMnemonic(KeyEvent.VK_W);
		window_group = new ButtonGroup();
		fileMenu.setMnemonic(KeyEvent.VK_F);
		
//		recent_menu = new JMenu("Recent Files");
//		recent_menu.setMnemonic(KeyEvent.VK_R);
//		createRecentMenu();
		recent_menu = createRecentMenu();
		
		JMenuItem closeAllMenuItem = new JMenuItem("Close All");
		closeAllMenuItem.setMnemonic(KeyEvent.VK_C);
		window_menu.add(closeAllMenuItem);
		window_menu.addSeparator();
		closeAllMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				closeAllWindows();
			}
		});
		JMenuItem newCatMenuItem = new JMenuItem("New Category");
		newCatMenuItem.setMnemonic(KeyEvent.VK_N);
		newCatMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				newCategoryInternalFrame();
			}
		});
		JMenuItem openCatMenuItem = new JMenuItem("Open Category");
		openCatMenuItem.setMnemonic(KeyEvent.VK_O);
		openCatMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				openCategory();
			}
		});
		JMenuItem dlCatMenuItem = new JMenuItem("Download Category");
		dlCatMenuItem.setMnemonic(KeyEvent.VK_D);
		dlCatMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadCategory();
			}
		});

		JMenuItem newFunMenuItem = new JMenuItem("New Functor");
		newFunMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					createFunctorInternalFrame();
				}
		});
		JMenuItem openFunMenuItem = new JMenuItem("Open Functor");
		openFunMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					openFunctor();
				}
		});

		JMenuItem dlFunMenuItem = new JMenuItem("Download Functor");
		dlFunMenuItem.setMnemonic(KeyEvent.VK_F);
		dlFunMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					loadFunctor();
				}
		});
		
		
		JMenuItem exitMenuItem = new JMenuItem("Exit");
		exitMenuItem.setMnemonic(KeyEvent.VK_X);
		exitMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					dispose();
					System.exit(0);
				}
		});   
		fileMenu.add(newCatMenuItem);
		fileMenu.add(openCatMenuItem);
		fileMenu.add(dlCatMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(newFunMenuItem);
		fileMenu.add(openFunMenuItem);
		fileMenu.add(dlFunMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(recent_menu);
		fileMenu.addSeparator();
		fileMenu.add(exitMenuItem);


		JMenu settingsMenu = new JMenu("Settings");
		settingsMenu.setMnemonic(KeyEvent.VK_S);
		JMenuItem setServerMenuItem = new JMenuItem("Set Server");
		setServerMenuItem.setMnemonic(KeyEvent.VK_S);
		setServerMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				set_server_action();
			}
		});
		
		JMenuItem setEndoMenuItem = new JMenuItem("Set Endomorphism Limit");
		setEndoMenuItem.setMnemonic(KeyEvent.VK_E);
		setEndoMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				set_endomorphism_limit_action();
			}
		});
		
		settingsMenu.add(setServerMenuItem);
		settingsMenu.add(setEndoMenuItem);
		
		JMenu helpMenu = new JMenu("Help");
		helpMenu.setMnemonic(KeyEvent.VK_H);
		JMenuItem helpTopicsMenuItem = new JMenuItem("Help Topics Index");
		helpTopicsMenuItem.setMnemonic(KeyEvent.VK_H);
		helpTopicsMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				helpTopicsMenuItem_actionPerformed(e);
			}
		});
		JMenuItem bugReportMenuItem = new JMenuItem("Bug Report");
		bugReportMenuItem.setMnemonic(KeyEvent.VK_B);
		bugReportMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				bugReportMenuItem_actionPerformed(e);
			}
		});
		JMenuItem aboutGDCTMenuItem = new JMenuItem("About GDCT");
		aboutGDCTMenuItem.setMnemonic(KeyEvent.VK_G);
		aboutGDCTMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				aboutGDCTMenuItem_actionPerformed(e);
			}
		});
		JMenuItem aboutAuthorsMenuItem = new JMenuItem("About Authors");
		aboutAuthorsMenuItem.setMnemonic(KeyEvent.VK_A);
		aboutAuthorsMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				aboutAuthorsMenuItem_actionPerformed(e);
			}
		});

		helpMenu.add(helpTopicsMenuItem);
		helpMenu.addSeparator();
		helpMenu.add(bugReportMenuItem);
		helpMenu.addSeparator();
		helpMenu.add(aboutGDCTMenuItem);
		helpMenu.add(aboutAuthorsMenuItem);


		menuBar.add(fileMenu);
		menuBar.add(settingsMenu);
		menuBar.add(window_menu);
		menuBar.add(helpMenu);

		return menuBar;
	}

	private void createWindowAction(JInternalFrame JIF)
	{
		final JRadioButtonMenuItem new_item = (JRadioButtonMenuItem)window_menu.add(new JRadioButtonMenuItem(JIF.getTitle()));
		window_group.add(new_item);
		new_item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				JInternalFrame[] jifs = desktop.getAllFrames();
				for (int i=0; i<jifs.length; i++)
				{
					if (jifs[i].getTitle() == new_item.getText())
					{
						try {jifs[i].setSelected(true);}
						catch (PropertyVetoException pve) {}
					}
				}
			}
		});
		new_item.setSelected(true);
	}
	
	private void closeWindowAction(JInternalFrame JIF)
	{
		Component[] menuItems = window_menu.getMenuComponents();
		for (int i=0; i<menuItems.length; i++)
		{
			if (menuItems[i] instanceof JMenuItem && ((JMenuItem)menuItems[i]).getText() == JIF.getTitle())
			{
				window_menu.remove((JMenuItem)menuItems[i]);
				return;
			}
		}
	}
	
	private void closeAllWindows()
	{
		JInternalFrame[] jifs = desktop.getAllFrames();
		for (int i=0; i<jifs.length; i++)
		{
			try {jifs[i].setClosed(true);}
			catch (PropertyVetoException pve) {}
		}
	}
	
	private void newCategoryInternalFrame()
	{
		CreateCategoryDialog ccdFrame = new CreateCategoryDialog(ini);
 		displayFrame(ccdFrame);
 	}
				
	public void createCategoryInternalFrame(Category newCategory_)
	{		
		CategoryInternalFrame frame = new CategoryInternalFrame(newCategory_);
		displayFrame(frame);
	}
   
	private void openCategory()
	{
		fchooser = new JFileChooser();
		fchooser.addChoosableFileFilter(new CatFilter());
		fchooser.addChoosableFileFilter(new CglFilter());
		fchooser.setAcceptAllFileFilterUsed(false);
		int returnVal = fchooser.showOpenDialog(this);
			
		if(returnVal == JFileChooser.APPROVE_OPTION)
		{
			
			File file = fchooser.getSelectedFile();
			OpenCategory open = new OpenCategory(file.getPath(),ini,true);
			Category cat = open.getCategory();
			
			cat.mod.isDownloaded();
			//now can create an internal Category Frame with this as a parameter
			createCategoryInternalFrame(cat);
			recent_menu = createRecentMenu();
		}
		else
		{
			//not wanting to open anything
		}
		
	}
   
	private void openFunctor()
	{
		fchooser = new JFileChooser();
		fchooser.addChoosableFileFilter(new FunFilter());
		fchooser.addChoosableFileFilter(new FglFilter());
		fchooser.setAcceptAllFileFilterUsed(false);
		
		int returnVal = fchooser.showOpenDialog(this);
		if(returnVal == JFileChooser.APPROVE_OPTION)
		{
			//get the file name and then open this
			File file = fchooser.getSelectedFile();
			
			//open fun file
			OpenFunctor open = new OpenFunctor(file.getPath(),ini,true);
	
			//then create an internalfunctorframe with the functor data
			FunctorNode fnode = open.getFunctor();//maybe have to use new here?
	
			//now can create an internalFunctorFrame with this as a parameter
			FunctorInternalFrame frame = new FunctorInternalFrame(fnode);
			displayFrame(frame);
			recent_menu = createRecentMenu();
		}
		else {}//do not open the file, got cancelled
	}
	
	private void loadCategory()
	{
		listData  = new Vector();
		
		try
		{
			cglURL = new URL(ini.getServer() + ini.getCglDir());
			catURL = new URL(ini.getServer() + ini.getCatDir());
		}
		catch (MalformedURLException malformedURL)
		{
			System.out.println("Error!  bad URL string!!");
		}
		
		getContents(cglURL);
		getContents(catURL);
		

		Object[] poss = listData.toArray();
		String ret = (String)JOptionPane.showInternalInputDialog(
			desktop,
			"Please select one of the following Categories",
			"Select Category to Download",
			JOptionPane.PLAIN_MESSAGE,
			new ImageIcon("image1.gif"),//doesn't really do anything
			poss,
			poss[0]);
			
		if(ret!= null)
		{		
			String dir = ret.substring(ret.length()-3).toLowerCase();
			
			if(dir == "cat")
			{
				dir = ini.getCatDir();
			}
			else if(dir == "cgl")
			{
				dir = ini.getCglDir();
			}
			String location= new String(ini.getServer()+dir+"/"+ret);
		
			//get the file from the server
			OpenCategory open = new OpenCategory(location,ini,false);
			
			Category cat = open.getCategory();
			cat.mod.isDownloaded();
			//now can create an internal Category Frame with this as a parameter
			createCategoryInternalFrame(cat);
			recent_menu = createRecentMenu();
		}
		else {}
	}
	
	private void loadFunctor()
	{	
		listData = new Vector();
			
		try
		{
			fglURL = new URL(ini.getServer() + ini.getFglDir());
			funURL = new URL(ini.getServer() + ini.getFunDir());
		}
		catch (MalformedURLException malformedURL)
		{
			System.out.println("Error!  bad URL string!!");
		}
		
		getContents(fglURL);//should check for return values
		getContents(funURL);		

		Object[] poss = listData.toArray();
		String s = (String)JOptionPane.showInternalInputDialog(
			desktop,
			"Please select one of the following functors",
			"Select Functor to Download",
			JOptionPane.PLAIN_MESSAGE,
			new ImageIcon("image1.gif"),
			poss,
			poss[0]);

		if(s != null)
		{	
			String dir = s.substring(s.length()-3).toLowerCase();		
			
			if(dir == "fun")
			{
				dir = ini.getFunDir();
			}
			else if(dir == "fgl")
			{
				dir = ini.getFglDir();
			}
			String location= new String(ini.getServer()+dir+"/"+s);
		
			//open fun file
			OpenFunctor open = new OpenFunctor(location,ini,false);
	
			//then create an internalfunctorframe with the functor data
	
			FunctorNode fnode = open.getFunctor();//maybe have to use new here?
			//now can create an internalFunctorFrame with this as a parameter
			FunctorInternalFrame frame = new FunctorInternalFrame(fnode);
			displayFrame(frame);
			recent_menu = createRecentMenu();
		}
		else {}

	}

	private boolean getContents(URL location){
		String text;
		StringTokenizer tokens;
		int linecount = 0;
		String temp;
		InputStream input;
		DataInputStream dataInput;
	
		try
		{
		input = location.openStream();
		dataInput = new DataInputStream(input);

		while((text=dataInput.readLine()) != null)
		{
			tokens = new StringTokenizer(text);
			while(tokens.hasMoreTokens())
			{
				temp = tokens.nextToken();
				if ((temp.charAt(0) == 'H') && (temp.charAt(1) == 'R'))
				{
					int i = 6;
					int j = 0;
					char catname[] = new char[50];
					String cname = new String();
					while ( temp.charAt(i) != '"')
					{
						catname[j] = temp.charAt(i);
						i++;
						j++;
					}
					cname = temp.substring(6, i);
					if (!cname.substring(0,1).equals("?") && !ini.getServer().endsWith(cname))
					{
					//add item to load list
					if(cname.toLowerCase().endsWith("fgl")
						|| cname.toLowerCase().endsWith("fun")
						||cname.toLowerCase().endsWith("cat") 
						||cname.toLowerCase().endsWith("cgl"))
						{
							listData.addElement(cname);
						}
					}
				}
			}
		}
		dataInput.close();
		}
		catch(IOException e)
		{
			JOptionPane.showMessageDialog(this, "Unable to connect to " + ini.getServer() + ini.getFglDir(), "Error", JOptionPane.ERROR_MESSAGE);
			return(false);
		}
		return(true);
	}

	private void createFunctorInternalFrame() {
		//get ready to create a functor		
		CreateFunctorDialog f = new CreateFunctorDialog(ini);
		//Test f = new Test();
		displayFrame(f);
	}
	
	public IniSettings getIni() {
		return ini;
	}

	private void set_server_action()
	{
		SettingsServer ss = new SettingsServer(ini);
		displayFrame(ss);
		ss.setLayer(JLayeredPane.MODAL_LAYER);
	}

	private void set_endomorphism_limit_action()
	{
		EndomorphismFrame ef = new EndomorphismFrame(ini);
		displayFrame(ef);
		ef.setLayer(JLayeredPane.MODAL_LAYER);
	}

	private void aboutGDCTMenuItem_actionPerformed(ActionEvent e)
	{
		AboutFrame a = new AboutFrame();
		displayFrame(a);
	}

	private void bugReportMenuItem_actionPerformed(ActionEvent e)
	{
		BugFrame b = new BugFrame();
		displayFrame(b);
	}

	private void aboutAuthorsMenuItem_actionPerformed(ActionEvent e)
	{
		AboutAuthors a = new AboutAuthors();
		displayFrame(a);
	}

	private void helpTopicsMenuItem_actionPerformed(ActionEvent e)
	{
		HelpFrame h = new HelpFrame();
	}

/*	private void menu_action(ActionEvent e)
	{		
		try
		{
			Class temp_class = Class.forName(((JMenuItem)e.getSource()).getText() + "Frame");
			System.out.println("class name: " + temp_class.getName());
			displayFrame((JInternalFrame)(temp_class.newInstance()));
		}
		catch (ClassNotFoundException cnfe) {cnfe.printStackTrace();}
		catch (InstantiationException ie) {ie.printStackTrace();}
		catch (IllegalAccessException iae) {iae.printStackTrace();}
	}
*/

	private JMenu createRecentMenu()
	{
	    ini.readRecentFiles();

		JMenu inside_recent_menu = new JMenu("Recent Files");
		JMenuItem[] recentMenuItems = new JMenuItem[8];

		String rname;
		for (int i=0; i<8; i++)
		{
			rname = ini.getRecent(i+1);
			recentMenuItems[i] = new JMenuItem(rname);
			if (!rname.equals(""))
			{
				recentMenuItems[i].addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						recent_menu_item_action(e.getActionCommand());
					}
				});
				inside_recent_menu.add(recentMenuItems[i]);
			}
		}
		return inside_recent_menu;
	}
	
	private void recent_menu_item_action(String rname_)
	{
		String rname = rname_; 
		if (rname.toLowerCase().endsWith(".cgl") || rname.toLowerCase().endsWith(".cat"))
		{
			Category cat;
			if (rname.toLowerCase().startsWith("http"))
			{				
				OpenCategory open = new OpenCategory(rname, ini, false);
				cat = open.getCategory();
				cat.mod.isDownloaded();
			}
			else
			{
				OpenCategory open = new OpenCategory(rname, ini, true);
				cat = open.getCategory();
				cat.mod.isCreated();
			}
			//now can create an internal Category Frame with this as a parameter
			createCategoryInternalFrame(cat);
		}
		else if (rname.toLowerCase().endsWith(".fgl") || rname.toLowerCase().endsWith(".fun"))
		{
			OpenFunctor open;
			if (rname.toLowerCase().startsWith("http"))
			{
				open = new OpenFunctor(rname, ini, false);
			}
			else
			{
				open = new OpenFunctor(rname, ini, true);
			}
			//then create an internalfunctorframe with the functor data
			FunctorNode fnode = open.getFunctor();//maybe have to use new here?
	
			//now can create an internalFunctorFrame with this as a parameter
			FunctorInternalFrame frame = new FunctorInternalFrame(fnode);
			displayFrame(frame);
		}
	}

	private void displayFrame(JInternalFrame JIF)
	{
		JIF.addInternalFrameListener(this);
		desktop.add(JIF);
        JIF.setLocation(new java.util.Random().nextInt(250) ,
		new java.util.Random().nextInt(100) );
		JIF.pack();
		JIF.show();
		createWindowAction(JIF);
		try{JIF.setSelected(true);} 
		catch (java.beans.PropertyVetoException e) {}
	}
	
	public void internalFrameActivated(InternalFrameEvent e) 
	{
		Component[] menuItems = window_menu.getMenuComponents();
		for (int i=0; i<menuItems.length; i++)
		{
			if (menuItems[i] instanceof JMenuItem && ((JMenuItem)menuItems[i]).getText() == ((JInternalFrame)e.getSource()).getTitle())
			{
				((JMenuItem)menuItems[i]).setSelected(true);
				return;
			}
		}
	}
	public void internalFrameClosed(InternalFrameEvent e) {}
	public void internalFrameClosing(InternalFrameEvent e) 
	{
		closeWindowAction((JInternalFrame)e.getSource());
	}
	public void internalFrameDeactivated(InternalFrameEvent e) {}
	public void internalFrameDeiconified(InternalFrameEvent e) {}
	public void internalFrameIconified(InternalFrameEvent e) {}
	public void internalFrameOpened(InternalFrameEvent e) {}

	private class AboutAuthors extends JInternalFrame
	{
		private JLabel image_;
		public AboutAuthors()
		{
			super("About GDCT Authors", false, true, false, false);
			JLabel image_ = new JLabel(new ImageIcon("images/authors.jpg"));
			getContentPane().add(image_);
			setSize(image_.getSize());
		}
	}

	private class AboutFrame extends JInternalFrame
	{
		private JLabel image_;
		private JTextArea text_;
		public AboutFrame()
		{
			super("About GDCT", false, true, false, false);
			image_ = new JLabel(new ImageIcon("images/small.gif"));
			text_ = new JTextArea(	"Graphical Database for Category Theory\n" +
									"Version 3.0, Developed 1998-2003\n\n" +
									"R. Rosebrugh, J. Bradbury, I. Rutherford,\n" +
									"M. Graves, J. Tweedle\n\n" +
									"Dept. of Mathematics & Computer Science\n" +
									"Mount Allison University, New Brunswick\n" +
									"Canada");
			text_.setFont(Font.getFont("Times New Roman"));
			text_.setBackground(this.getBackground());
			getContentPane().setLayout(new GridBagLayout());
			getContentPane().add(image_,  new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
					    ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
			getContentPane().add(text_,  new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0
					    ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
			setSize(image_.getWidth() + text_.getWidth(), image_.getHeight());
		}
	}
	
	private class BugFrame extends JInternalFrame
	{
		JLabel image_;
		JTextArea text_;
		
		public BugFrame()
		{
			super("GDCT: Bug Report", false, true, false, false);
			image_ = new JLabel(new ImageIcon("images/bug.jpg"));
			text_ = new JTextArea(	"\nand there are still a few bugs and problems that have\n" +
									"not been corrected.\n\n\n" +
									"If you notice any abnormal behavior or incorrect\n" +
									"behavior on the part of the program please check to\n" +
									"see if this is a known bug on the GDCT webpage at\n" +
									"http://mathcs.mta.ca/research/rosebrugh/gdct/bugs.htm\n" +
									"If you do not see your problem listed please click\n" +
									"the Report Bug button on that page and fill out\n" +
									"the provided online form.\n\n\n" +
									"Thank you for helping to improve the quality of\n" +
									"the Graphical Database for Category Theory\n" +
									"software.");
			text_.setFont(new Font("Times New Roman", Font.PLAIN, 14));
			text_.setBackground(this.getBackground());
			getContentPane().setLayout(new GridBagLayout());
			getContentPane().add(image_, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
			,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
			getContentPane().add(text_, new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0
			,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
			setSize(image_.getWidth() + text_.getWidth(), image_.getHeight());
		}
	}
}