/*
Research Project:   Graphical Database for Category Theory
                    J. Bradbury, Dr. R. Rosebrugh, I. Rutherford
                    Mount Allison University 2001
                    Updated: May 8th, 2003 (J. Tweedle)
File:               PullbackFrame.java
Description:        This class implements a frame which allows the user to run
                    the category tool "Pullback" on the current category.  The
                    algorithms used to test the category for sums were
                    removed and put into the file Pullback.java
*/

//import statements
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;

public class PullbackFrame extends JInternalFrame
{
	private CategoryInternalFrame categoryInternalFrame_;
	private Category cat = new Category();
	private Pullback pullback;
	private int obj;
	private String objchr;

//Objects in the Graphical User Interface(GUI)
	private JLabel objectLabel = new JLabel();
	private JLabel alphaLabel = new JLabel();
	private JLabel betaLabel = new JLabel();
	private JLabel lambdaLabel = new JLabel();
	private JLabel rhoLabel = new JLabel();
	private JButton okButton = new JButton();
	private JButton pullbackButton = new JButton();
	private JComboBox objectChoice = new JComboBox();
	private JCheckBox lambdarhoChoice = new JCheckBox();
	private JTextField alphaField = new JTextField(13);
	private JTextField betaField = new JTextField(13);
	private JTextField lambdaField = new JTextField(13);
	private JTextField rhoField = new JTextField(13);

	public PullbackFrame(CategoryInternalFrame categoryInternalFrame)
	//Default Constructor with the parent frame as a parameter
	{
		super(categoryInternalFrame.getTitle() + ": Pullback", true, true, false, false);
		categoryInternalFrame_ = categoryInternalFrame;
		cat.copyCategory(categoryInternalFrame_.getCategory());
		pullback = new Pullback(cat, "pullback");

		okButton.setText("Close");
		objectLabel.setText("Pullback Object:");
		alphaLabel.setText("Alpha:");
		betaLabel.setText("Beta:");
		lambdaLabel.setText("Lambda:");
		lambdaLabel.setEnabled(false);
		rhoLabel.setText("Rho:");
		rhoLabel.setEnabled(false);
		lambdarhoChoice.setText("Specify lambda/rho");
		objectChoice.setBackground(Color.white);

		pullbackButton.setText("Pullback?");
		alphaField.setBackground(Color.white);
		betaField.setBackground(Color.white);
		lambdaField.setBackground(Color.white);
		lambdaField.setEnabled(false);
		rhoField.setBackground(Color.white);
		rhoField.setEnabled(false);


		pullbackButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				pullbackButton_actionPerformed();
			}
		});

		objectChoice.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				objectChoice_itemStateChanged(e);
			}
		});

		okButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				okButton_actionPerformed(e);
			}
		});
		lambdarhoChoice.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				lambda_rho_action();
			}
		});

		Container contentPane = getContentPane();
		contentPane.setLayout(new GridBagLayout());
		contentPane.add(pullbackButton,	new GridBagConstraints(0, 6, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(okButton, new GridBagConstraints(1, 6, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(lambdaLabel, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(rhoLabel, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(alphaLabel, new GridBagConstraints(0, 3, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(betaLabel, new GridBagConstraints(0, 4, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(objectLabel, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(objectChoice, new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(lambdaField, new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(rhoField, new GridBagConstraints(1, 2, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(alphaField, new GridBagConstraints(1, 3, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(betaField, new GridBagConstraints(1, 4, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(lambdarhoChoice, new GridBagConstraints(0, 5, 2, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		
		objectChoice.addItem("Check all objects");
		for (int i = 0; i < cat.num_objects; i++)
			objectChoice.addItem(cat.obj[i]);
		objectChoice.setSelectedIndex(1);
	}
	
	private void pullbackButton_actionPerformed()
	{
		pullback.setNumPullbacks(0);
		pullback.setOutput("");
		String viewbox = "";
		pullback.setEndoPassed(false);
		
		objchr = (String)objectChoice.getSelectedItem();
		if (objchr.equals("Check all objects"))
		{
			if (!cat.check_obj(lambdaField.getText()))
			{
				JOptionPane.showInternalMessageDialog(this, lambdaField.getText() + " is not an object of " + cat.name + ".\n", "Error", JOptionPane.ERROR_MESSAGE);
				return;
			}
			else if (!cat.check_obj(rhoField.getText()))
			{
				JOptionPane.showInternalMessageDialog(this, rhoField.getText() + " is not an object of " + cat.name + ".\n", "Error", JOptionPane.ERROR_MESSAGE);
				return;
			}
			else if (!cat.check_obj(alphaField.getText()))
			{
				JOptionPane.showInternalMessageDialog(this, alphaField.getText() + " is not an object of " + cat.name + ".\n", "Error", JOptionPane.ERROR_MESSAGE);
				return;
			}
			else
			{
				for (int i=0; i<cat.num_objects; i++)
				{
					objchr = (String)objectChoice.getItemAt(i+1);
					pullback.pullbackAllObjects(lambdaField.getText(), rhoField.getText(), alphaField.getText(), objchr);
				}
				if (pullback.getNumPullbacks() == 0)
				{
					JOptionPane.showInternalMessageDialog(this, "There is no pullback of objects " + lambdaField.getText() + " and " + rhoField.getText() + ".", "No Pullbacks Found", JOptionPane.ERROR_MESSAGE);
					return;
				}
				else if (pullback.getNumPullbacks() == 1)
					viewbox += pullback.getOutput() + "1 pullback was found.\n";
				else
					viewbox += pullback.getOutput() + Integer.toString(pullback.getNumPullbacks()) + " pullbacks were found.\n";
			}
		}
		else if (lambdarhoChoice.isSelected())
		{
			int test = pullback.pullbackOneObject(alphaField.getText(), betaField.getText(), lambdaField.getText(), rhoField.getText(), objchr);
			int message_type = JOptionPane.PLAIN_MESSAGE;
			if (test == Pullback.ERROR)
			{
				message_type = JOptionPane.ERROR_MESSAGE;
			}
			JOptionPane.showInternalMessageDialog(this, pullback.getOutput(), "Pullbacks", message_type);
			return;
		}
		else
		{
			int numpullbacks = pullback.pullbackOneObjectNoLambdaRho(alphaField.getText(), betaField.getText(), objchr);
			int message_type = JOptionPane.PLAIN_MESSAGE;
			if (numpullbacks == Pullback.ERROR)
			{
				message_type = JOptionPane.ERROR_MESSAGE;
			}
			JOptionPane.showInternalMessageDialog(this, pullback.getOutput() + "\n" + numpullbacks + 
			" pullback(s) found.", "Pullbacks", message_type);
			return;
		}
		
		if (pullback.getEndoPassed())
			viewbox+="WARNING: Endomorpmism limit reached! Results may be inaccurate!";
		JOptionPane.showInternalMessageDialog(this, viewbox, "Pullbacks", JOptionPane.PLAIN_MESSAGE);		
	}
	
	private void okButton_actionPerformed(ActionEvent e)
	//Exit sum frame and return to main frame
	{
		try {this.setClosed(true);}
		catch (PropertyVetoException PVE) {}
	}
	
	private void lambda_rho_action()
	{
		lambdaLabel.setEnabled(lambdarhoChoice.isSelected());
		rhoLabel.setEnabled(lambdarhoChoice.isSelected());
		lambdaField.setEnabled(lambdarhoChoice.isSelected());
		rhoField.setEnabled(lambdarhoChoice.isSelected());
	}

	private void objectChoice_itemStateChanged (ItemEvent e)
	{
		if (objectChoice.getSelectedItem().equals("Check all objects"))
		{
			lambdaLabel.setEnabled(true);
			lambdaField.setEnabled(true);
			rhoLabel.setEnabled(true);
			rhoField.setEnabled(true);
			lambdaLabel.setText("Domain Object 1 (A):");
			rhoLabel.setText("Domain Object 2 (B):");
			alphaLabel.setText("Codomain Object (C):");
			betaLabel.setEnabled(false);
			betaField.setEnabled(false);
			lambdarhoChoice.setSelected(false);
			lambdarhoChoice.setEnabled(false);
		}
		else
		{
			lambdaLabel.setEnabled(false);
			lambdaField.setEnabled(false);
			rhoLabel.setEnabled(false);
			rhoField.setEnabled(false);
			lambdaLabel.setText("Lambda:");
			rhoLabel.setText("Rho:");
			alphaLabel.setText("Alpha:");
			betaLabel.setEnabled(true);
			betaField.setEnabled(true);	
			lambdarhoChoice.setEnabled(true);
		}
	}
	
}