/*
Research Project:   Graphical Database for Category Theory
                    J. Bradbury, Dr. R. Rosebrugh, I. Rutherford
                    Mount Allison University 2001
					Created: May 8th, 2003 (J. Tweedle)
File:               PushoutFrame.java
Description:        This class implements a frame which allows the user to run
                    the category tool "Pushout" on the current category.  The
                    algorithms used to test the category for sums were
                    removed and put into the file Pullback.java
*/

//import statements
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;

public class PushoutFrame extends JInternalFrame
{
	private CategoryInternalFrame categoryInternalFrame_;
	private Category cat = new Category();
	private Pullback pushout;
	private int obj;
	private String objchr;

//Objects in the Graphical User Interface(GUI)
	private JLabel objectLabel = new JLabel();
	private JLabel alphaLabel = new JLabel();
	private JLabel betaLabel = new JLabel();
	private JLabel lambdaLabel = new JLabel();
	private JLabel rhoLabel = new JLabel();
	private JButton okButton = new JButton();
	private JButton pushoutButton = new JButton();
	private JComboBox objectChoice = new JComboBox();
	private JCheckBox lambdarhoChoice = new JCheckBox();
	private JTextField alphaField = new JTextField(13);
	private JTextField betaField = new JTextField(13);
	private JTextField lambdaField = new JTextField(13);
	private JTextField rhoField = new JTextField(13);

	public PushoutFrame(CategoryInternalFrame categoryInternalFrame)
	//Default Constructor with the parent frame as a parameter
	{
		super(categoryInternalFrame.getTitle() + ": Pushout", true, true, false, false);
		categoryInternalFrame_ = categoryInternalFrame;
		cat = categoryInternalFrame_.getCategory();
		pushout = new Pullback(cat.makeDual(), "pushout");

		okButton.setText("Close");
		objectLabel.setText("Pushout Object:");
		alphaLabel.setText("Alpha:");
		betaLabel.setText("Beta:");
		lambdaLabel.setText("Lambda:");
		lambdaLabel.setEnabled(false);
		rhoLabel.setText("Rho:");
		rhoLabel.setEnabled(false);
		lambdarhoChoice.setText("Specify lambda/rho");
		objectChoice.setBackground(Color.white);

		pushoutButton.setText("Pushout?");
		alphaField.setBackground(Color.white);
		betaField.setBackground(Color.white);
		lambdaField.setBackground(Color.white);
		lambdaField.setEnabled(false);
		rhoField.setBackground(Color.white);
		rhoField.setEnabled(false);


		pushoutButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				pushoutButton_actionPerformed();
			}
		});

		objectChoice.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				objectChoice_itemStateChanged(e);
			}
		});

		okButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				okButton_actionPerformed(e);
			}
		});
		lambdarhoChoice.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				lambda_rho_action();
			}
		});


		Container contentPane = getContentPane();
		contentPane.setLayout(new GridBagLayout());
		contentPane.add(pushoutButton,	new GridBagConstraints(0, 6, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(okButton, new GridBagConstraints(1, 6, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(lambdaLabel, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(rhoLabel, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(alphaLabel, new GridBagConstraints(0, 3, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(betaLabel, new GridBagConstraints(0, 4, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(objectLabel, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(objectChoice, new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(lambdaField, new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(rhoField, new GridBagConstraints(1, 2, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(alphaField, new GridBagConstraints(1, 3, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(betaField, new GridBagConstraints(1, 4, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(lambdarhoChoice, new GridBagConstraints(0, 5, 2, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));
		
		
		objectChoice.addItem("Check all objects");
		for (int i = 0; i < cat.num_objects; i++)
			objectChoice.addItem(cat.obj[i]);
		objectChoice.setSelectedIndex(1);
	}
	
	private void pushoutButton_actionPerformed()
	{
		pushout.setNumPullbacks(0);
		pushout.setOutput("");
		String viewbox = "";
		pushout.setEndoPassed(false);
		
		objchr = (String)objectChoice.getSelectedItem();
		if (objchr.equals("Check all objects"))
		{
			if (!cat.check_obj(lambdaField.getText()))
			{
				JOptionPane.showInternalMessageDialog(this, lambdaField.getText() + " is not an object of " + cat.name + ".\n", "Error", JOptionPane.ERROR_MESSAGE);
				return;
			}
			else if (!cat.check_obj(rhoField.getText()))
			{
				JOptionPane.showInternalMessageDialog(this, rhoField.getText() + " is not an object of " + cat.name + ".\n", "Error", JOptionPane.ERROR_MESSAGE);
				return;
			}
			else if (!cat.check_obj(alphaField.getText()))
			{
				JOptionPane.showInternalMessageDialog(this, alphaField.getText() + " is not an object of " + cat.name + ".\n", "Error", JOptionPane.ERROR_MESSAGE);
				return;
			}
			else
			{
				for (int i=0; i<cat.num_objects; i++)
				{
					objchr = (String)objectChoice.getItemAt(i+1);
					pushout.pullbackAllObjects(lambdaField.getText(), rhoField.getText(), alphaField.getText(), objchr);
				}
				if (pushout.getNumPullbacks() == 0)
				{
					JOptionPane.showInternalMessageDialog(this, "There is no pushout of objects " + lambdaField.getText() + " and " + rhoField.getText() + ".", "No Pushouts Found", JOptionPane.ERROR_MESSAGE);
					return;
				}
				else if (pushout.getNumPullbacks() == 1)
					viewbox += pushout.getOutput() + "1 pushout was found.\n";
				else
					viewbox += pushout.getOutput() + Integer.toString(pushout.getNumPullbacks()) + " pushouts were found.\n";
			}
		}
		else if (lambdarhoChoice.isSelected())
		{
			String alpha = cat.path_to_string(cat.reverse_path(cat.string_to_path(alphaField.getText())));
			String beta = cat.path_to_string(cat.reverse_path(cat.string_to_path(betaField.getText())));
			String lambda = cat.path_to_string(cat.reverse_path(cat.string_to_path(lambdaField.getText())));
			String rho = cat.path_to_string(cat.reverse_path(cat.string_to_path(rhoField.getText())));
			int test = pushout.pullbackOneObject(alpha, beta, lambda, rho, objchr);
			int message_type = JOptionPane.PLAIN_MESSAGE;
			if (test == Pullback.ERROR)
			{
				message_type = JOptionPane.ERROR_MESSAGE;
			}
			JOptionPane.showInternalMessageDialog(this, pushout.getOutput(), "Pushouts", message_type);
			return;
		}
		else
		{
			String alpha = cat.path_to_string(cat.reverse_path(cat.string_to_path(alphaField.getText())));
			String beta = cat.path_to_string(cat.reverse_path(cat.string_to_path(betaField.getText())));
			int numpushouts = pushout.pullbackOneObjectNoLambdaRho(alpha, beta, objchr);
			if (numpushouts == Pullback.ERROR)
			{
				JOptionPane.showInternalMessageDialog(this, pushout.getOutput() +
				"\n0 pushouts found.", "Pushouts", JOptionPane.ERROR_MESSAGE);
				return;
			}
			JOptionPane.showInternalMessageDialog(this, pushout.getOutput() + "\n" + numpushouts + 
			" pushout(s) found.", "Pushouts", JOptionPane.PLAIN_MESSAGE);
			return;
		}
		if (pushout.getEndoPassed())
			viewbox+="WARNING: Endomorpmism limit reached! Results may be inaccurate!";
		JOptionPane.showInternalMessageDialog(this, viewbox, "Pushouts", JOptionPane.PLAIN_MESSAGE);		
	}
	
	private void okButton_actionPerformed(ActionEvent e)
	//Exit sum frame and return to main frame
	{
		try {this.setClosed(true);}
		catch (PropertyVetoException PVE) {}
	}

	private void lambda_rho_action()
	{
		lambdaLabel.setEnabled(lambdarhoChoice.isSelected());
		rhoLabel.setEnabled(lambdarhoChoice.isSelected());
		lambdaField.setEnabled(lambdarhoChoice.isSelected());
		rhoField.setEnabled(lambdarhoChoice.isSelected());
	}

	private void objectChoice_itemStateChanged (ItemEvent e)
	{
		if (objectChoice.getSelectedItem().equals("Check all objects"))
		{
			lambdaLabel.setEnabled(true);
			lambdaField.setEnabled(true);
			rhoLabel.setEnabled(true);
			rhoField.setEnabled(true);
			lambdaLabel.setText("Codomain Object 1 (A):");
			rhoLabel.setText("Codomain Object 2 (B):");
			alphaLabel.setText("Domain Object (C):");
			betaLabel.setEnabled(false);
			betaField.setEnabled(false);
			lambdarhoChoice.setSelected(false);
			lambdarhoChoice.setEnabled(false);
		}
		else
		{
			lambdaLabel.setEnabled(false);
			lambdaField.setEnabled(false);
			rhoLabel.setEnabled(false);
			rhoField.setEnabled(false);
			lambdaLabel.setText("Lambda:");
			rhoLabel.setText("Rho:");
			alphaLabel.setText("Alpha:");
			betaLabel.setEnabled(true);
			betaField.setEnabled(true);	
			lambdarhoChoice.setEnabled(true);
		}
	}
}